/*
 * Copyright 2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.relaxNG.compact;

import org.intellij.plugins.relaxNG.compact.lexer.CompactSyntaxLexerAdapter;

import com.intellij.lexer.Lexer;
import com.intellij.openapi.editor.HighlighterColors;
import com.intellij.openapi.editor.colors.CodeInsightColors;
import com.intellij.openapi.editor.colors.TextAttributesKey;
import com.intellij.openapi.fileTypes.SyntaxHighlighterBase;
import com.intellij.psi.tree.IElementType;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: sweinreuter
 * Date: 04.08.2007
 */
public class RncHighlighter extends SyntaxHighlighterBase {
  @NotNull
  public Lexer getHighlightingLexer() {
    return new CompactSyntaxLexerAdapter();
  }

  private static final Map<IElementType, TextAttributesKey> ourMap1;

  static {
    ourMap1 = new HashMap<IElementType, TextAttributesKey>();

    fillMap(ourMap1, RncTokenTypes.KEYWORDS, HighlighterColors.JAVA_KEYWORD);
    fillMap(ourMap1, RncTokenTypes.OPERATORS, HighlighterColors.JAVA_OPERATION_SIGN);

    fillMap(ourMap1, RncTokenTypes.STRINGS, HighlighterColors.JAVA_STRING);

    ourMap1.put(RncTokenTypes.LBRACE, HighlighterColors.JAVA_BRACES);
    ourMap1.put(RncTokenTypes.RBRACE, HighlighterColors.JAVA_BRACES);

    ourMap1.put(RncTokenTypes.LBRACKET, HighlighterColors.JAVA_BRACKETS);
    ourMap1.put(RncTokenTypes.RBRACKET, HighlighterColors.JAVA_BRACKETS);

    ourMap1.put(RncTokenTypes.LPAREN, HighlighterColors.JAVA_PARENTHS);
    ourMap1.put(RncTokenTypes.RPAREN, HighlighterColors.JAVA_PARENTHS);

    ourMap1.put(RncTokenTypes.COMMA, HighlighterColors.JAVA_COMMA);

    fillMap(ourMap1, RncTokenTypes.DOC_TOKENS, HighlighterColors.JAVA_DOC_COMMENT);
    fillMap(ourMap1, RncTokenTypes.COMMENTS, HighlighterColors.JAVA_LINE_COMMENT);

    fillMap(ourMap1, RncTokenTypes.IDENTIFIERS, CodeInsightColors.LOCAL_VARIABLE_ATTRIBUTES);

    ourMap1.put(RncTokenTypes.ILLEGAL_CHAR, HighlighterColors.BAD_CHARACTER);
  }

  @NotNull
  public TextAttributesKey[] getTokenHighlights(IElementType tokenType) {
    return pack(ourMap1.get(tokenType));
  }
}